@@ -20,6 +20,7 @@
 
 from __future__ import annotations
 
+import contextlib
 import typing
 import warnings
 import weakref
@@ -30,6 +31,7 @@ from urwid.util import (
     apply_target_encoding,
     calc_text_pos,
     calc_width,
+    get_encoding,
     rle_append_modify,
     rle_join_modify,
     rle_len,
@@ -38,7 +40,7 @@ from urwid.util import (
 )
 
 if typing.TYPE_CHECKING:
-    from collections.abc import Sequence
+    from collections.abc import Hashable, Iterable, Sequence
 
     from .widget import Widget
 
@@ -72,7 +74,7 @@ class CanvasCache:
     _refs: typing.ClassVar[
         dict[
             weakref.ReferenceType,
-            tuple[type[Widget], tuple[int, int] | tuple[int] | tuple[()], bool],
+            tuple[Widget, type[Widget], tuple[int, int] | tuple[int] | tuple[()], bool],
         ]
     ] = {}
     _deps: typing.ClassVar[dict[Widget, list[Widget]]] = {}
@@ -152,13 +154,12 @@ class CanvasCache:
         """
         Remove all canvases cached for widget.
         """
-        try:
+        with contextlib.suppress(KeyError):
             for ref in cls._widgets[widget].values():
                 with suppress(KeyError):
                     del cls._refs[ref]
             del cls._widgets[widget]
-        except KeyError:
-            pass
+
         if widget not in cls._deps:
             return
         dependants = cls._deps.get(widget, [])
@@ -182,11 +183,9 @@ class CanvasCache:
         with suppress(KeyError):
             del sizes[(wcls, size, focus)]
         if not sizes:
-            try:
+            with contextlib.suppress(KeyError):
                 del cls._widgets[widget]
                 del cls._deps[widget]
-            except KeyError:
-                pass
 
     @classmethod
     def clear(cls):
@@ -229,8 +228,8 @@ class Canvas:
         if value1 is not None:
             raise self._renamed_error
         self._widget_info = None
-        self.coords = {}
-        self.shortcuts = {}
+        self.coords: dict[str, tuple[int, int, tuple[Widget, int, int]]] = {}
+        self.shortcuts: dict[str, str] = {}
 
     def finalize(
         self,
@@ -269,11 +268,16 @@ class Canvas:
     @property
     def text(self) -> list[bytes]:
         """
-        Return the text content of the canvas as a list of strings,
-        one for each row.
+        Return the text content of the canvas as a list of strings, one for each row.
         """
         return [b"".join([text for (attr, cs, text) in row]) for row in self.content()]
 
+    @property
+    def decoded_text(self) -> Sequence[str]:
+        """Decoded text content of the canvas as a sequence of strings, one for each row."""
+        encoding = get_encoding()
+        return tuple(line.decode(encoding) for line in self.text)
+
     def _text_content(self):
         warnings.warn(
             f"Method `{self.__class__.__name__}._text_content` is deprecated, "
@@ -290,7 +294,7 @@ class Canvas:
         cols: int | None = None,
         rows: int | None = None,
         attr=None,
-    ):
+    ) -> Iterable[list[tuple[object, object, bytes]]]:
         raise NotImplementedError()
 
     def cols(self):
@@ -299,7 +303,7 @@ class Canvas:
     def rows(self):
         raise NotImplementedError()
 
-    def content_delta(self):
+    def content_delta(self, other: Canvas):
         raise NotImplementedError()
 
     def get_cursor(self) -> tuple[int, int] | None:
@@ -308,7 +312,7 @@ class Canvas:
             return None
         return c[:2]  # trim off data part
 
-    def set_cursor(self, c):
+    def set_cursor(self, c: tuple[int, int] | None) -> None:
         if self.widget_info and self.cacheable:
             raise self._finalized_error
         if c is None:
@@ -319,13 +323,13 @@ class Canvas:
 
     cursor = property(get_cursor, set_cursor)
 
-    def get_pop_up(self):
+    def get_pop_up(self) -> tuple[int, int, tuple[Widget, int, int]] | None:
         c = self.coords.get("pop up", None)
         if not c:
             return None
         return c
 
-    def set_pop_up(self, w: Widget, left: int, top: int, overlay_width: int, overlay_height: int):
+    def set_pop_up(self, w: Widget, left: int, top: int, overlay_width: int, overlay_height: int) -> None:
         """
         This method adds pop-up information to the canvas.  This information
         is intercepted by a PopUpTarget widget higher in the chain to
@@ -348,7 +352,7 @@ class Canvas:
 
         self.coords["pop up"] = (left, top, (w, overlay_width, overlay_height))
 
-    def translate_coords(self, dx: int, dy: int):
+    def translate_coords(self, dx: int, dy: int) -> dict[str, tuple[int, int, tuple[Widget, int, int]]]:
         """
         Return coords shifted by (dx, dy).
         """
@@ -357,6 +361,25 @@ class Canvas:
             d[name] = (x + dx, y + dy, data)
         return d
 
+    def __repr__(self) -> str:
+        extra = [""]
+        with contextlib.suppress(BaseException):
+            extra.append(f"cols={self.cols()}")
+
+        with contextlib.suppress(BaseException):
+            extra.append(f"rows={self.rows()}")
+
+        if self.cursor:
+            extra.append(f"cursor={self.cursor}")
+
+        return f"<{self.__class__.__name__} finalized={bool(self.widget_info)}{' '.join(extra)} at 0x{id(self):X}>"
+
+    def __str__(self) -> str:
+        with contextlib.suppress(BaseException):
+            return "\n".join(self.decoded_text)
+
+        return repr(self)
+
 
 class TextCanvas(Canvas):
     """
@@ -388,7 +411,10 @@ class TextCanvas(Canvas):
             widths = []
             for t in text:
                 if not isinstance(t, bytes):
-                    raise CanvasError("Canvas text must be plain strings encoded in the screen's encoding", repr(text))
+                    raise CanvasError(
+                        "Canvas text must be plain strings encoded in the screen's encoding",
+                        repr(text),
+                    )
                 widths.append(calc_width(t, 0, len(t)))
         else:
             if not isinstance(maxcol, int):
@@ -411,7 +437,13 @@ class TextCanvas(Canvas):
         for i in range(len(text)):
             w = widths[i]
             if w > maxcol:
-                raise CanvasError(f"Canvas text is wider than the maxcol specified \n{maxcol!r}\n{widths!r}\n{text!r}")
+                raise CanvasError(
+                    f"Canvas text is wider than the maxcol specified:\n"
+                    f"maxcol={maxcol!r}\n"
+                    f"widths={widths!r}\n"
+                    f"text={text!r}\n"
+                    f"urwid target encoding={get_encoding()}"
+                )
             if w < maxcol:
                 text[i] += b"".rjust(maxcol - w)
             a_gap = len(text[i]) - rle_len(attr[i])
@@ -457,10 +489,10 @@ class TextCanvas(Canvas):
         self,
         trim_left: int = 0,
         trim_top: int = 0,
-        cols: int = 0,
-        rows: int = 0,
-        attr_map=None,
-    ):
+        cols: int | None = 0,
+        rows: int | None = 0,
+        attr=None,
+    ) -> Iterable[tuple[object, object, bytes]]:
         """
         Return the canvas content as a list of rows where each row
         is a list of (attr, cs, text) tuples.
@@ -502,13 +534,13 @@ class TextCanvas(Canvas):
             i = 0
             row = []
             for (a, cs), run in attr_cs:
-                if attr_map and a in attr_map:
-                    a = attr_map[a]  # noqa: PLW2901
+                if attr and a in attr:
+                    a = attr[a]  # noqa: PLW2901
                 row.append((a, cs, text[i : i + run]))
                 i += run
             yield row
 
-    def content_delta(self, other):
+    def content_delta(self, other: Canvas):
         """
         Return the differences between other and this canvas.
 
@@ -534,10 +566,10 @@ class BlankCanvas(Canvas):
         self,
         trim_left: int = 0,
         trim_top: int = 0,
-        cols: int = 0,
-        rows: int = 0,
+        cols: int | None = 0,
+        rows: int | None = 0,
         attr=None,
-    ):
+    ) -> Iterable[list[tuple[object, object, bytes]]]:
         """
         return (cols, rows) of spaces with default attributes.
         """
@@ -554,7 +586,7 @@ class BlankCanvas(Canvas):
     def rows(self) -> typing.NoReturn:
         raise NotImplementedError("BlankCanvas doesn't know its own size!")
 
-    def content_delta(self) -> typing.NoReturn:
+    def content_delta(self, other: Canvas) -> typing.NoReturn:
         raise NotImplementedError("BlankCanvas doesn't know its own size!")
 
 
@@ -589,7 +621,7 @@ class SolidCanvas(Canvas):
         cols: int | None = None,
         rows: int | None = None,
         attr=None,
-    ):
+    ) -> Iterable[list[tuple[object, object, bytes]]]:
         if cols is None:
             cols = self.size[0]
         if rows is None:
@@ -636,8 +668,13 @@ class CompositeCanvas(Canvas):
         super().__init__()
 
         if canv is None:
-            self.shards = []
-            self.children = []
+            self.shards: list[
+                tuple[
+                    int,
+                    list[tuple[int, int, int, int, dict[Hashable | None, Hashable] | None, Canvas]],
+                ]
+            ] = []
+            self.children: list[tuple[int, int, Canvas, typing.Any]] = []
         else:
             if hasattr(canv, "shards"):
                 self.shards = canv.shards
@@ -648,6 +685,21 @@ class CompositeCanvas(Canvas):
             for shortcut in canv.shortcuts:
                 self.shortcuts[shortcut] = "wrap"
 
+    def __repr__(self) -> str:
+        extra = [""]
+        with contextlib.suppress(BaseException):
+            extra.append(f"cols={self.cols()}")
+
+        with contextlib.suppress(BaseException):
+            extra.append(f"rows={self.rows()}")
+
+        if self.cursor:
+            extra.append(f"cursor={self.cursor}")
+        if self.children:
+            extra.append(f"children=({', '.join(repr(canv) for _, _, canv, _ in self.children)})")
+
+        return f"<{self.__class__.__name__} finalized={bool(self.widget_info)}{' '.join(extra)} at 0x{id(self):X}>"
+
     def rows(self) -> int:
         for r, cv in self.shards:
             if not isinstance(r, int):
@@ -660,12 +712,19 @@ class CompositeCanvas(Canvas):
     def cols(self) -> int:
         if not self.shards:
             return 0
-        cols = sum([cv[2] for cv in self.shards[0][1]])
+        cols = sum(cv[2] for cv in self.shards[0][1])
         if not isinstance(cols, int):
             raise TypeError(cols)
         return cols
 
-    def content(self):
+    def content(
+        self,
+        trim_left: int = 0,
+        trim_top: int = 0,
+        cols: int | None = None,
+        rows: int | None = None,
+        attr=None,
+    ) -> Iterable[list[tuple[object, object, bytes]]]:
         """
         Return the canvas content as a list of rows where each row
         is a list of (attr, cs, text) tuples.
@@ -682,7 +741,7 @@ class CompositeCanvas(Canvas):
             # prepare next shard tail
             shard_tail = shard_body_tail(num_rows, sbody)
 
-    def content_delta(self, other):
+    def content_delta(self, other: Canvas):
         """
         Return the differences between other and this canvas.
         """
@@ -798,7 +857,7 @@ class CompositeCanvas(Canvas):
                 self.shards = self.shards[:]
             self.shards.append((bottom, [(0, 0, cols, bottom, None, blank_canvas)]))
 
-    def overlay(self, other, left: int, top: int) -> None:
+    def overlay(self, other: CompositeCanvas, left: int, top: int) -> None:
         """Overlay other onto this canvas."""
         if self.widget_info:
             raise self._finalized_error
@@ -842,14 +901,14 @@ class CompositeCanvas(Canvas):
 
         self.coords.update(other.translate_coords(left, top))
 
-    def fill_attr(self, a) -> None:
+    def fill_attr(self, a: Hashable) -> None:
+        """
+        Apply attribute a to all areas of this canvas with default attribute currently set to None,
+        leaving other attributes intact.
         """
-        Apply attribute a to all areas of this canvas with default
-        attribute currently set to None, leaving other attributes
-        intact."""
         self.fill_attr_apply({None: a})
 
-    def fill_attr_apply(self, mapping) -> None:
+    def fill_attr_apply(self, mapping: dict[Hashable | None, Hashable]) -> None:
         """
         Apply an attribute-mapping dictionary to the canvas.
 
@@ -910,7 +969,7 @@ def shard_body_tail(num_rows: int, sbody):
     """
     shard_tail = []
     col_gap = 0
-    done_rows = 0
+
     for done_rows, content_iter, cview in sbody:
         cols, rows = cview[2:4]
         done_rows += num_rows  # noqa: PLW2901
@@ -1036,9 +1095,8 @@ def shards_trim_top(shards, top: int):
     sbody = shard_body(cviews, shard_tail, False)
     shard_tail = shard_body_tail(num_rows, sbody)
     # trim the top of this shard
-    new_sbody = []
-    for done_rows, content_iter, cv in sbody:
-        new_sbody.append((0, content_iter, cview_trim_top(cv, done_rows + top)))
+    new_sbody = [(0, content_iter, cview_trim_top(cv, done_rows + top)) for done_rows, content_iter, cv in sbody]
+
     sbody = new_sbody
 
     new_shards = [(num_rows - top, [cv for done_rows, content_iter, cv in sbody])]
@@ -1125,7 +1183,7 @@ def shards_join(shard_lists):
     new_shards = []
     while True:
         new_cviews = []
-        num_rows = min([r for r, cv in shards_current])
+        num_rows = min(r for r, cv in shards_current)
 
         shards_next = []
         for rows, cviews in shards_current:
@@ -1163,7 +1221,7 @@ def cview_trim_cols(cv, cols: int):
     return cv[:2] + (cols,) + cv[3:]
 
 
-def CanvasCombine(canvas_info):
+def CanvasCombine(canvas_info: Iterable[tuple[Canvas, typing.Any, bool]]) -> CompositeCanvas:
     """Stack canvases in l vertically and return resulting canvas.
 
     :param canvas_info: list of (canvas, position, focus) tuples:
@@ -1180,8 +1238,8 @@ def CanvasCombine(canvas_info):
     children = []
     row = 0
     focus_index = 0
-    n = 0
-    for canv, pos, focus in clist:
+
+    for n, (canv, pos, focus) in enumerate(clist):
         if focus:
             focus_index = n
         children.append((0, row, canv, pos))
@@ -1190,7 +1248,6 @@ def CanvasCombine(canvas_info):
         for shortcut in canv.shortcuts:
             combined_canvas.shortcuts[shortcut] = pos
         row += canv.rows()
-        n += 1
 
     if focus_index:
         children = [children[focus_index]] + children[:focus_index] + children[focus_index + 1 :]
@@ -1200,7 +1257,7 @@ def CanvasCombine(canvas_info):
     return combined_canvas
 
 
-def CanvasOverlay(top_c, bottom_c, left: int, top: int):
+def CanvasOverlay(top_c, bottom_c, left: int, top: int) -> CompositeCanvas:
     """
     Overlay canvas top_c onto bottom_c at position (left, top).
     """
@@ -1213,7 +1270,7 @@ def CanvasOverlay(top_c, bottom_c, left: int, top: int):
     return overlayed_canvas
 
 
-def CanvasJoin(canvas_info):
+def CanvasJoin(canvas_info: Iterable[tuple[Canvas, typing.Any, bool, int]]) -> CompositeCanvas:
     """
     Join canvases in l horizontally. Return result.
 
@@ -1232,8 +1289,8 @@ def CanvasJoin(canvas_info):
     l2 = []
     focus_item = 0
     maxrow = 0
-    n = 0
-    for canv, pos, focus, cols in canvas_info:
+
+    for n, (canv, pos, focus, cols) in enumerate(canvas_info):
         rows = canv.rows()
         pad_right = cols - canv.cols()
         if focus:
@@ -1241,7 +1298,6 @@ def CanvasJoin(canvas_info):
         if rows > maxrow:
             maxrow = rows
         l2.append((canv, pos, pad_right, rows))
-        n += 1
 
     shard_lists = []
     children = []
@@ -1268,7 +1324,7 @@ def CanvasJoin(canvas_info):
     return joined_canvas
 
 
-def apply_text_layout(text, attr, ls, maxcol: int):
+def apply_text_layout(text: bytes, attr, ls, maxcol: int) -> TextCanvas:
     t = []
     a = []
     c = []
