@@ -295,12 +295,10 @@ class Screen(BaseScreen, RealTerminal):
         self.resize_tenths = convert_to_tenths(resize_wait)
 
     @typing.overload
-    def get_input(self, raw_keys: Literal[False]) -> list[str]:
-        ...
+    def get_input(self, raw_keys: Literal[False]) -> list[str]: ...
 
     @typing.overload
-    def get_input(self, raw_keys: Literal[True]) -> tuple[list[str], list[int]]:
-        ...
+    def get_input(self, raw_keys: Literal[True]) -> tuple[list[str], list[int]]: ...
 
     def get_input(self, raw_keys: bool = False) -> list[str] | tuple[list[str], list[int]]:
         """Return pending input as a list.
@@ -575,8 +573,8 @@ class Screen(BaseScreen, RealTerminal):
 
             first = True
             lasta = None
-            nr = 0
-            for a, cs, seg in row:
+
+            for nr, (a, cs, seg) in enumerate(row):
                 if cs != "U":
                     seg = seg.translate(UNPRINTABLE_TRANS_TABLE)  # noqa: PLW2901
                     if not isinstance(seg, bytes):
@@ -602,7 +600,7 @@ class Screen(BaseScreen, RealTerminal):
                         # perhaps screen size changed
                         # quietly abort.
                         return
-                nr += 1
+
         if r.cursor is not None:
             x, y = r.cursor
             self._curs_set(1)
