@@ -25,7 +25,7 @@ import warnings
 from pprint import pformat
 
 from urwid.canvas import CanvasError, TextCanvas
-from urwid.escape import SAFE_ASCII_DEC_SPECIAL_RE
+from urwid.display.escape import SAFE_ASCII_DEC_SPECIAL_RE
 from urwid.util import apply_target_encoding, str_util
 
 if typing.TYPE_CHECKING:
@@ -131,14 +131,16 @@ class FontRegistry(type):
         """Get font by name if registered.
 
         This method is needed to get access to font from registry class.
+        >>> from urwid.util import set_temporary_encoding
         >>> repr(FontRegistry["a"])
         'None'
         >>> font = FontRegistry["Thin 3x3"]()
         >>> font.height
         3
-        >>> canvas: TextCanvas = font.render("+")
-        >>> b'\\n'.join(canvas.text).decode('utf-8') == "  \\n ┼\\n  "
-        True
+        >>> with set_temporary_encoding("utf-8"):
+        ...     canvas: TextCanvas = font.render("+")
+        >>> b'\\n'.join(canvas.text).decode('utf-8')
+        '  \\n ┼\\n  '
         """
         return cls.__registered.get(item)
 
@@ -152,7 +154,7 @@ class FontRegistry(type):
         """List of (font name, font class) tuples."""
         return list(mcs.__registered.items())
 
-    def __new__(  # noqa: PYI034  # new can not return Self
+    def __new__(
         cls: type[FontRegistry],
         name: str,
         bases: tuple[type, ...],
@@ -688,6 +690,6 @@ if __name__ == "__main__":
         if font_chars == all_ascii:
             print("Full ASCII")
         elif font_chars & all_ascii == all_ascii:
-            print(f"Full ASCII + {''.join(font_chars^all_ascii)!r}")
+            print(f"Full ASCII + {''.join(font_chars ^ all_ascii)!r}")
         else:
             print(f"Characters: {chars!r}")
