@@ -24,11 +24,14 @@ from __future__ import annotations
 import atexit
 import copy
 import errno
+import fcntl
 import os
-import select
+import pty
+import selectors
 import signal
 import struct
 import sys
+import termios
 import time
 import traceback
 import typing
@@ -37,19 +40,14 @@ from collections import deque
 from contextlib import suppress
 from dataclasses import dataclass
 
-try:
-    import fcntl
-    import pty
-    import termios
-except ImportError:
-    pass  # windows
-
 from urwid import event_loop, util
 from urwid.canvas import Canvas
-from urwid.display_common import _BASIC_COLORS, AttrSpec, RealTerminal, _color_desc_256, _color_desc_true
-from urwid.escape import ALT_DEC_SPECIAL_CHARS, DEC_SPECIAL_CHARS
+from urwid.display import AttrSpec, RealTerminal
+from urwid.display.escape import ALT_DEC_SPECIAL_CHARS, DEC_SPECIAL_CHARS
 from urwid.widget import Sizing, Widget
 
+from .display.common import _BASIC_COLORS, _color_desc_256, _color_desc_true
+
 if typing.TYPE_CHECKING:
     from collections.abc import Callable, Iterable, Mapping, Sequence
 
@@ -543,7 +541,7 @@ class TermCanvas(Canvas):
         qmark = self.escbuf.startswith(b"?")
 
         escbuf = []
-        for arg in self.escbuf[qmark and 1 or 0 :].split(b";"):
+        for arg in self.escbuf[1 if qmark else 0 :].split(b";"):
             try:
                 num = int(arg)
             except ValueError:
@@ -586,7 +584,7 @@ class TermCanvas(Canvas):
             elif char in b"G8":
                 # 8 is obsolete and only for backwards compatibility
                 self.modes.main_charset = CHARSET_UTF8
-        elif mod in (b"(", b")"):  # define G0/G1
+        elif mod in {b"(", b")"}:  # define G0/G1
             self.set_g01(char, mod)
         elif char == b"M":  # reverse line feed
             self.linefeed(reverse=True)
@@ -609,7 +607,7 @@ class TermCanvas(Canvas):
         """
         Parse operating system command.
         """
-        if buf.startswith(b";") or buf.startswith(b"0;") or buf.startswith(b"2;"):
+        if buf.startswith((b";", b"0;", b"2;")):
             # set window title
             self.widget.set_title(buf.decode().partition(";")[2])
 
@@ -647,14 +645,14 @@ class TermCanvas(Canvas):
             self.escbuf = b""
             self.parsestate = 1
             return
-        elif self.parsestate == 0 and char in (b"%", b"#", b"(", b")"):
+        elif self.parsestate == 0 and char in {b"%", b"#", b"(", b")"}:
             # non-CSI sequence
             self.escbuf = char
             self.parsestate = 3
             return
         elif self.parsestate == 3:
             self.parse_noncsi(char, self.escbuf)
-        elif char in (b"c", b"D", b"E", b"H", b"M", b"Z", b"7", b"8", b">", b"="):
+        elif char in {b"c", b"D", b"E", b"H", b"M", b"Z", b"7", b"8", b">", b"="}:
             self.parse_noncsi(char)
 
         self.leave_escape()
@@ -702,7 +700,7 @@ class TermCanvas(Canvas):
                 # end multibyte sequence
                 self.utf8_eat_bytes = None
                 sequence = (self.utf8_buffer + bytes([byte])).decode("utf-8", "ignore")
-                if len(sequence) == 0:
+                if not sequence:
                     # invalid multibyte sequence, stop processing
                     return
                 char = sequence.encode(util._target_encoding, "replace")
@@ -1120,7 +1118,7 @@ class TermCanvas(Canvas):
             elif 40 <= attr <= 47:
                 bg = attr - 40
                 colors = max(16, colors)
-            elif attr in (38, 48):
+            elif attr in {38, 48}:
                 if idx + 2 < len(attrs) and attrs[idx + 1] == 5:
                     # 8 bit color specification
                     color = attrs[idx + 2]
@@ -1148,7 +1146,7 @@ class TermCanvas(Canvas):
             elif attr == 10:
                 self.charset.reset_sgr_ibmpc()
                 self.modes.display_ctrl = False
-            elif attr in (11, 12):
+            elif attr in {11, 12}:
                 self.charset.set_sgr_ibmpc()
                 self.modes.display_ctrl = True
 
@@ -1192,10 +1190,10 @@ class TermCanvas(Canvas):
         fg = _defaulter(fg, colors)
         bg = _defaulter(bg, colors)
 
-        if len(attributes) > 0:
+        if attributes:
             fg = ",".join((fg, *list(attributes)))
 
-        if fg == "default" and bg == "default":
+        if fg == bg == "default":
             return None
 
         if colors:
@@ -1439,18 +1437,18 @@ class TermCanvas(Canvas):
     def content(
         self,
         trim_left: int = 0,
-        trim_right: int = 0,
+        trim_top: int = 0,
         cols: int | None = None,
         rows: int | None = None,
         attr=None,
-    ):
+    ) -> Iterable[list[tuple[object, object, bytes]]]:
         if self.scrolling_up == 0:
             yield from self.term
         else:
             buf = self.scrollback_buffer + self.term
             yield from buf[-(self.height + self.scrolling_up) : -self.scrolling_up]
 
-    def content_delta(self, other):
+    def content_delta(self, other: Canvas):
         if other is self:
             return [self.cols()] * self.rows()
         return self.content()
@@ -1464,7 +1462,7 @@ class Terminal(Widget):
 
     def __init__(
         self,
-        command: Sequence[str | bytes] | Callable[[], ...] | None,
+        command: Sequence[str | bytes] | Callable[[], typing.Any] | None,
         env: Mapping[str, str] | Iterable[tuple[str, str]] | None = None,
         main_loop: event_loop.EventLoop | None = None,
         escape_sequence: str | None = None,
@@ -1562,6 +1560,7 @@ class Terminal(Widget):
         if self.pid == 0:
             if callable(self.command):
                 try:
+                    # noinspection PyBroadException
                     try:
                         self.command()
                     except BaseException:  # special case
@@ -1590,7 +1589,7 @@ class Terminal(Widget):
             for sig in (signal.SIGHUP, signal.SIGCONT, signal.SIGINT, signal.SIGTERM, signal.SIGKILL):
                 try:
                     os.kill(self.pid, sig)
-                    pid, status = os.waitpid(self.pid, os.WNOHANG)
+                    pid, _status = os.waitpid(self.pid, os.WNOHANG)
                 except OSError:
                     break
 
@@ -1670,7 +1669,7 @@ class Terminal(Widget):
         elif hasattr(self, "old_tios"):
             RealTerminal().tty_signal_keys(*self.old_tios)
 
-    def render(self, size: tuple[int, int], focus: bool = False):
+    def render(self, size: tuple[int, int], focus: bool = False) -> TermCanvas:
         if not self.terminated:
             self.change_focus(focus)
 
@@ -1693,13 +1692,11 @@ class Terminal(Widget):
         self.main_loop.remove_watch_file(self.master)
 
     def wait_and_feed(self, timeout: float = 1.0) -> None:
-        while True:
-            try:
-                select.select([self.master], [], [], timeout)
-                break
-            except OSError as e:
-                if e.args[0] != 4:
-                    raise
+        with selectors.DefaultSelector() as selector:
+            selector.register(self.master, selectors.EVENT_READ)
+
+            selector.select(timeout)
+
         self.feed()
 
     def feed(self) -> None:
@@ -1728,7 +1725,7 @@ class Terminal(Widget):
         if self.terminated:
             return key
 
-        if key in ("begin paste", "end paste"):
+        if key in {"begin paste", "end paste"}:
             if self.term_modes.bracketed_paste:
                 pass  # passthrough bracketed paste sequences
             else:  # swallow bracketed paste sequences
@@ -1740,7 +1737,7 @@ class Terminal(Widget):
             self.touch_term(width, height)
             return None
 
-        if self.last_key == self.escape_sequence and key == self.escape_sequence:
+        if self.last_key == key == self.escape_sequence:
             # escape sequence pressed twice...
             self.last_key = key
             self.keygrab = True
