@@ -5,9 +5,9 @@ import typing
 from urwid.canvas import CanvasCombine, CompositeCanvas, SolidCanvas
 from urwid.util import get_encoding_mode
 
-from .constants import Sizing
+from .constants import BAR_SYMBOLS, Sizing
 from .text import Text
-from .widget import Widget, WidgetMeta, nocache_widget_render, nocache_widget_render_instance
+from .widget import Widget, WidgetError, WidgetMeta, nocache_widget_render, nocache_widget_render_instance
 
 if typing.TYPE_CHECKING:
     from typing_extensions import Literal
@@ -40,7 +40,7 @@ def nocache_bargraph_get_data(self, get_data_fn):
     self._get_data = get_data_fn
 
 
-class BarGraphError(Exception):
+class BarGraphError(WidgetError):
     pass
 
 
@@ -49,15 +49,15 @@ class BarGraph(Widget, metaclass=BarGraphMeta):
 
     ignore_focus = True
 
-    eighths = " ▁▂▃▄▅▆▇"
+    eighths = BAR_SYMBOLS.VERTICAL[:8]  # Full height is done by style
     hlines = "_⎺⎻─⎼⎽"
 
-    def __init__(self, attlist, hatt=None, satt=None):
+    def __init__(self, attlist, hatt=None, satt=None) -> None:
         """
         Create a bar graph with the passed display characteristics.
         see set_segment_attributes for a description of the parameters.
         """
-
+        super().__init__()
         self.set_segment_attributes(attlist, hatt, satt)
         self.set_data([], 1, None)
         self.set_bar_width(None)
@@ -134,7 +134,7 @@ class BarGraph(Widget, metaclass=BarGraphMeta):
                 raise BarGraphError(f"fg ({fg}) not > bg ({bg})")
         self.satt = satt
 
-    def set_data(self, bardata, top, hlines=None):
+    def set_data(self, bardata, top: float, hlines=None) -> None:
         """
         Store bar data, bargraph top and horizontal line positions.
 
@@ -428,7 +428,7 @@ class BarGraph(Widget, metaclass=BarGraphMeta):
         return canv
 
 
-def calculate_bargraph_display(bardata, top, bar_widths, maxrow: int):
+def calculate_bargraph_display(bardata, top: float, bar_widths: list[int], maxrow: int):
     """
     Calculate a rendering of the bar graph described by data, bar_widths
     and height.
@@ -574,7 +574,7 @@ def calculate_bargraph_display(bardata, top, bar_widths, maxrow: int):
 class GraphVScale(Widget):
     _sizing = frozenset([Sizing.BOX])
 
-    def __init__(self, labels, top):
+    def __init__(self, labels, top: float) -> None:
         """
         GraphVScale( [(label1 position, label1 markup),...], top )
         label position -- 0 < position < top for the y position
@@ -584,9 +584,10 @@ class GraphVScale(Widget):
         This widget is a vertical scale for the BarGraph widget that
         can correspond to the BarGraph's horizontal lines
         """
+        super().__init__()
         self.set_scale(labels, top)
 
-    def set_scale(self, labels, top):
+    def set_scale(self, labels, top: float) -> None:
         """
         set_scale( [(label1 position, label1 markup),...], top )
         label position -- 0 < position < top for the y position
@@ -609,7 +610,7 @@ class GraphVScale(Widget):
         """
         return False
 
-    def render(self, size: tuple[int, int], focus: bool = False):
+    def render(self, size: tuple[int, int], focus: bool = False) -> SolidCanvas | CompositeCanvas:
         """
         Render GraphVScale.
         """
@@ -640,7 +641,7 @@ class GraphVScale(Widget):
         return c
 
 
-def scale_bar_values(bar, top, maxrow: int):
+def scale_bar_values(bar, top: float, maxrow: int) -> list[int]:
     """
     Return a list of bar values aliased to integer values of maxrow.
     """
