@@ -126,7 +126,7 @@ class Edit(Text):
             + ["multiline"] * (self.multiline is True)
         )
 
-    def _repr_attrs(self):
+    def _repr_attrs(self) -> dict[str, typing.Any]:
         attrs = dict(super()._repr_attrs(), edit_pos=self._edit_pos)
         return remove_defaults(attrs, Edit.__init__)
 
@@ -550,7 +550,7 @@ class Edit(Text):
         self._invalidate()
         return True
 
-    def mouse_event(self, size: tuple[int], event, button: int, x: int, y: int, focus: bool) -> bool:
+    def mouse_event(self, size: tuple[int], event: str, button: int, x: int, y: int, focus: bool) -> bool | None:
         """
         Move the cursor to the location clicked for button 1.
 
