@@ -17,33 +17,35 @@ from .constants import (
     simplify_height,
     simplify_valign,
 )
-from .widget_decoration import WidgetDecoration
+from .widget_decoration import WidgetDecoration, WidgetError
 
 if typing.TYPE_CHECKING:
     from typing_extensions import Literal
 
-    from .widget import Widget
+
+WrappedWidget = typing.TypeVar("WrappedWidget")
 
 
-class FillerError(Exception):
+class FillerError(WidgetError):
     pass
 
 
-class Filler(WidgetDecoration):
+class Filler(WidgetDecoration[WrappedWidget]):
     def __init__(
         self,
-        body: Widget,
+        body: WrappedWidget,
         valign: (
             Literal["top", "middle", "bottom"] | VAlign | tuple[Literal["relative", WHSettings.RELATIVE], int]
         ) = VAlign.MIDDLE,
-        height: int | Literal["pack"] | tuple[Literal["relative"], int] | None = WHSettings.PACK,
+        height: (
+            int | Literal["pack", WHSettings.PACK] | tuple[Literal["relative", WHSettings.RELATIVE], int] | None
+        ) = WHSettings.PACK,
         min_height: int | None = None,
         top: int = 0,
         bottom: int = 0,
     ) -> None:
         """
-        :param body: a flow widget or box widget to be filled around (stored
-            as self.original_widget)
+        :param body: a flow widget or box widget to be filled around (stored as self.original_widget)
         :type body: Widget
 
         :param valign: one of:
@@ -74,12 +76,13 @@ class Filler(WidgetDecoration):
         :param bottom: a fixed number of rows to fill at the bottom
         :type bottom: int
 
-        If body is a flow widget then height must be ``'flow'`` and
-        *min_height* will be ignored.
+        If body is a flow widget, then height must be ``'pack'`` and *min_height* will be ignored.
+        Sizing of the filler will be BOX/FLOW in this case.
 
-        Filler widgets will try to satisfy height argument first by
-        reducing the valign amount when necessary.  If height still
-        cannot be satisfied it will also be reduced.
+        If height is integer, *min_height* will be ignored and sizing of filler will be BOX/FLOW.
+
+        Filler widgets will try to satisfy height argument first by reducing the valign amount when necessary.
+        If height still cannot be satisfied it will also be reduced.
         """
         super().__init__(body)
 
@@ -95,13 +98,22 @@ class Filler(WidgetDecoration):
                     raise FillerError("fixed bottom height may only be used with fixed top valign")
                 bottom = height[1]
                 height = RELATIVE_100
+
         if isinstance(valign, tuple):
             if valign[0] == "fixed top":
                 top = valign[1]
-                valign = VAlign.TOP
+                normalized_valign = VAlign.TOP
             elif valign[0] == "fixed bottom":
                 bottom = valign[1]
-                valign = VAlign.BOTTOM
+                normalized_valign = VAlign.BOTTOM
+            else:
+                normalized_valign = valign
+
+        elif not isinstance(valign, (VAlign, str)):
+            raise FillerError(f"invalid valign: {valign!r}")
+
+        else:
+            normalized_valign = VAlign(valign)
 
         # convert old flow mode parameter height=None to height='flow'
         if height is None or height == Sizing.FLOW:
@@ -109,7 +121,7 @@ class Filler(WidgetDecoration):
 
         self.top = top
         self.bottom = bottom
-        self.valign_type, self.valign_amount = normalize_valign(valign, FillerError)
+        self.valign_type, self.valign_amount = normalize_valign(normalized_valign, FillerError)
         self.height_type, self.height_amount = normalize_height(height, FillerError)
 
         if self.height_type not in {WHSettings.GIVEN, WHSettings.PACK}:
@@ -117,10 +129,26 @@ class Filler(WidgetDecoration):
         else:
             self.min_height = None
 
-    def sizing(self):
-        return {Sizing.BOX}  # always a box widget
+    def sizing(self) -> frozenset[Sizing]:
+        """Widget sizing.
 
-    def _repr_attrs(self):
+        Sizing BOX is always supported.
+        Sizing FLOW is supported if: FLOW widget (a height type is PACK) or BOX widget with height GIVEN
+        """
+        sizing: set[Sizing] = {Sizing.BOX}
+        if self.height_type in {WHSettings.PACK, WHSettings.GIVEN}:
+            sizing.add(Sizing.FLOW)
+        return frozenset(sizing)
+
+    def rows(self, size: tuple[int], focus: bool = False) -> int:
+        """Flow pack support if FLOW sizing supported."""
+        if self.height_type == WHSettings.PACK:
+            return self.original_widget.rows(size, focus) + self.top + self.bottom
+        if self.height_type == WHSettings.GIVEN:
+            return self.height_amount + self.top + self.bottom
+        raise FillerError("Method 'rows' not supported for BOX widgets")  # pragma: no cover
+
+    def _repr_attrs(self) -> dict[str, typing.Any]:
         attrs = dict(
             super()._repr_attrs(),
             valign=simplify_valign(self.valign_type, self.valign_amount),
@@ -132,7 +160,7 @@ class Filler(WidgetDecoration):
         return remove_defaults(attrs, Filler.__init__)
 
     @property
-    def body(self):
+    def body(self) -> WrappedWidget:
         """backwards compatibility, widget used to be stored as body"""
         warnings.warn(
             "backwards compatibility, widget used to be stored as body",
@@ -142,7 +170,7 @@ class Filler(WidgetDecoration):
         return self.original_widget
 
     @body.setter
-    def body(self, new_body):
+    def body(self, new_body: WrappedWidget) -> None:
         warnings.warn(
             "backwards compatibility, widget used to be stored as body",
             PendingDeprecationWarning,
@@ -150,7 +178,7 @@ class Filler(WidgetDecoration):
         )
         self.original_widget = new_body
 
-    def get_body(self):
+    def get_body(self) -> WrappedWidget:
         """backwards compatibility, widget used to be stored as body"""
         warnings.warn(
             "backwards compatibility, widget used to be stored as body",
@@ -159,7 +187,7 @@ class Filler(WidgetDecoration):
         )
         return self.original_widget
 
-    def set_body(self, new_body):
+    def set_body(self, new_body: WrappedWidget) -> None:
         warnings.warn(
             "backwards compatibility, widget used to be stored as body",
             DeprecationWarning,
@@ -171,18 +199,25 @@ class Filler(WidgetDecoration):
         """Return selectable from body."""
         return self._original_widget.selectable()
 
-    def filler_values(self, size: tuple[int, int], focus: bool) -> tuple[int, int]:
+    def filler_values(self, size: tuple[int, int] | tuple[int], focus: bool) -> tuple[int, int]:
         """
         Return the number of rows to pad on the top and bottom.
 
         Override this method to define custom padding behaviour.
         """
-        (maxcol, maxrow) = size
+        maxcol, maxrow = self.pack(size, focus)
 
         if self.height_type == WHSettings.PACK:
             height = self._original_widget.rows((maxcol,), focus=focus)
             return calculate_top_bottom_filler(
-                maxrow, self.valign_type, self.valign_amount, WHSettings.GIVEN, height, None, self.top, self.bottom
+                maxrow,
+                self.valign_type,
+                self.valign_amount,
+                WHSettings.GIVEN,
+                height,
+                None,
+                self.top,
+                self.bottom,
             )
 
         return calculate_top_bottom_filler(
@@ -196,9 +231,9 @@ class Filler(WidgetDecoration):
             self.bottom,
         )
 
-    def render(self, size: tuple[int, int], focus: bool = False) -> CompositeCanvas:
+    def render(self, size: tuple[int, int] | tuple[int], focus: bool = False) -> CompositeCanvas:
         """Render self.original_widget with space above and/or below."""
-        (maxcol, maxrow) = size
+        maxcol, maxrow = self.pack(size, focus)
         top, bottom = self.filler_values(size, focus)
 
         if self.height_type == WHSettings.PACK:
@@ -217,18 +252,18 @@ class Filler(WidgetDecoration):
         canv.pad_trim_top_bottom(top, bottom)
         return canv
 
-    def keypress(self, size: tuple[int, int], key: str) -> str | None:
+    def keypress(self, size: tuple[int, int] | tuple[()], key: str) -> str | None:
         """Pass keypress to self.original_widget."""
-        (maxcol, maxrow) = size
+        maxcol, maxrow = self.pack(size, True)
         if self.height_type == WHSettings.PACK:
             return self._original_widget.keypress((maxcol,), key)
 
         top, bottom = self.filler_values((maxcol, maxrow), True)
         return self._original_widget.keypress((maxcol, maxrow - top - bottom), key)
 
-    def get_cursor_coords(self, size: tuple[int, int]) -> tuple[int, int] | None:
+    def get_cursor_coords(self, size: tuple[int, int] | tuple[int]) -> tuple[int, int] | None:
         """Return cursor coords from self.original_widget if any."""
-        (maxcol, maxrow) = size
+        maxcol, maxrow = self.pack(size, True)
         if not hasattr(self._original_widget, "get_cursor_coords"):
             return None
 
@@ -244,9 +279,9 @@ class Filler(WidgetDecoration):
             y = maxrow - 1
         return x, y + top
 
-    def get_pref_col(self, size: tuple[int, int]) -> int | None:
+    def get_pref_col(self, size: tuple[int, int] | tuple[int]) -> int | None:
         """Return pref_col from self.original_widget if any."""
-        (maxcol, maxrow) = size
+        maxcol, maxrow = self.pack(size, True)
         if not hasattr(self._original_widget, "get_pref_col"):
             return None
 
@@ -258,9 +293,9 @@ class Filler(WidgetDecoration):
 
         return x
 
-    def move_cursor_to_coords(self, size: tuple[int, int], col: int, row: int) -> bool:
+    def move_cursor_to_coords(self, size: tuple[int, int] | tuple[int], col: int, row: int) -> bool:
         """Pass to self.original_widget."""
-        (maxcol, maxrow) = size
+        maxcol, maxrow = self.pack(size, True)
         if not hasattr(self._original_widget, "move_cursor_to_coords"):
             return True
 
@@ -274,15 +309,15 @@ class Filler(WidgetDecoration):
 
     def mouse_event(
         self,
-        size: tuple[int, int],
+        size: tuple[int, int] | tuple[int],
         event,
         button: int,
         col: int,
         row: int,
         focus: bool,
-    ) -> bool:
+    ) -> bool | None:
         """Pass to self.original_widget."""
-        (maxcol, maxrow) = size
+        maxcol, maxrow = self.pack(size, focus)
         if not hasattr(self._original_widget, "mouse_event"):
             return False
 
