@@ -27,7 +27,7 @@ import warnings
 from operator import attrgetter
 
 from urwid import signals
-from urwid.canvas import CanvasCache, CompositeCanvas
+from urwid.canvas import Canvas, CanvasCache, CompositeCanvas
 from urwid.command_map import command_map
 from urwid.split_repr import split_repr
 from urwid.util import MetaSuper
@@ -37,7 +37,7 @@ from .constants import Sizing
 if typing.TYPE_CHECKING:
     from collections.abc import Callable, Hashable
 
-
+WrappedWidget = typing.TypeVar("WrappedWidget")
 LOGGER = logging.getLogger(__name__)
 
 
@@ -172,7 +172,7 @@ def cache_widget_rows(cls):
     fn = cls.rows
 
     @functools.wraps(fn)
-    def cached_rows(self, size, focus=False):
+    def cached_rows(self, size: tuple[int], focus: bool = False) -> int:
         focus = focus and not ignore_focus
         canv = CanvasCache.fetch(self, cls, size, focus)
         if canv:
@@ -204,58 +204,6 @@ class Widget(metaclass=WidgetMeta):
        A shared :class:`CommandMap` instance. May be redefined
        in subclasses or widget instances.
 
-    .. method:: render(size, focus=False)
-
-       .. note::
-
-          This method is not implemented in :class:`.Widget` but
-          must be implemented by any concrete subclass
-
-       :param size: One of the following,
-                    *maxcol* and *maxrow* are integers > 0:
-
-                    (*maxcol*, *maxrow*)
-                      for box sizing -- the parent chooses the exact
-                      size of this widget
-
-                    (*maxcol*,)
-                      for flow sizing -- the parent chooses only the
-                      number of columns for this widget
-
-                    ()
-                      for fixed sizing -- this widget is a fixed size
-                      which can't be adjusted by the parent
-       :type size: widget size
-       :param focus: set to ``True`` if this widget or one of its children
-                     is in focus
-       :type focus: bool
-
-       :returns: A :class:`Canvas` subclass instance containing the
-                 rendered content of this widget
-
-       :class:`Text` widgets return a :class:`TextCanvas` (arbitrary text and
-       display attributes), :class:`SolidFill` widgets return a
-       :class:`SolidCanvas` (a single character repeated across
-       the whole surface) and container widgets return a
-       :class:`CompositeCanvas` (one or more other canvases
-       arranged arbitrarily).
-
-       If *focus* is ``False``, the returned canvas may not have a cursor
-       position set.
-
-       There is some metaclass magic defined in the :class:`Widget`
-       metaclass :class:`WidgetMeta` that causes the
-       result of this method to be cached by :class:`CanvasCache`.
-       Later calls will automatically look up the value in the cache first.
-
-       As a small optimization the class variable :attr:`ignore_focus`
-       may be defined and set to ``True`` if this widget renders the same
-       canvas regardless of the value of the *focus* parameter.
-
-       Any time the content of a widget changes it should call
-       :meth:`_invalidate` to remove any cached canvases, or the widget
-       may render the cached canvas instead of creating a new one.
-
 
     .. method:: rows(size, focus=False)
 
@@ -284,58 +232,6 @@ class Widget(metaclass=WidgetMeta):
        widget renders the same size regardless of the value of the *focus*
        parameter.
 
-
-    .. method:: keypress(size, key)
-
-       :param size: See :meth:`Widget.render` for details
-       :type size: widget size
-       :param key: a single keystroke value; see :ref:`keyboard-input`
-       :type key: str
-
-       :returns: ``None`` if *key* was handled by this widget or
-                 *key* (the same value passed) if *key* was not handled
-                 by this widget
-
-       Container widgets will typically call the :meth:`keypress` method on
-       whichever of their children is set as the focus.
-
-       The standard widgets use :attr:`_command_map` to
-       determine what action should be performed for a given *key*. You may
-       modify these values to your liking globally, at some level in the
-       widget hierarchy or on individual widgets. See :class:`CommandMap`
-       for the defaults.
-
-       In your own widgets you may use whatever logic you like: filtering or
-       translating keys, selectively passing along events etc.
-
-
-
-    .. method:: mouse_event(size, event, button, col, row, focus)
-
-       :param size: See :meth:`Widget.render` for details.
-       :type size: widget size
-       :param event: Values such as ``'mouse press'``, ``'ctrl mouse press'``,
-                     ``'mouse release'``, ``'meta mouse release'``,
-                     ``'mouse drag'``; see :ref:`mouse-input`
-       :type event: mouse event
-       :param button: 1 through 5 for press events, often 0 for release events
-                      (which button was released is often not known)
-       :type button: int
-       :param col: Column of the event, 0 is the left edge of this widget
-       :type col: int
-       :param row: Row of the event, 0 it the top row of this widget
-       :type row: int
-       :param focus: Set to ``True`` if this widget or one of its children
-                     is in focus
-       :type focus: bool
-
-       :returns: ``True`` if the event was handled by this widget, ``False``
-                 otherwise
-
-       Container widgets will typically call the :meth:`mouse_event` method on
-       whichever of their children is at the position (*col*, *row*).
-
-
     .. method:: get_cursor_coords(size)
 
        .. note::
@@ -420,14 +316,14 @@ class Widget(metaclass=WidgetMeta):
         """
         CanvasCache.invalidate(self)
 
-    def _emit(self, name: Hashable, *args):
+    def _emit(self, name: Hashable, *args) -> None:
         """
         Convenience function to emit signals with self as first
         argument.
         """
         signals.emit_signal(self, name, self, *args)
 
-    def selectable(self):
+    def selectable(self) -> bool:
         """
         :returns: ``True`` if this is a widget that is designed to take the
                   focus, i.e. it contains something the user might want to
@@ -448,7 +344,7 @@ class Widget(metaclass=WidgetMeta):
         """
         return self._selectable
 
-    def sizing(self):
+    def sizing(self) -> frozenset[Sizing]:
         """
         :returns: A frozenset including one or more of ``'box'``, ``'flow'`` and
                   ``'fixed'``.  Default implementation returns the value of
@@ -559,7 +455,7 @@ class Widget(metaclass=WidgetMeta):
         """
         return split_repr(self)
 
-    def _repr_words(self):
+    def _repr_words(self) -> list[str]:
         words = []
         if self.selectable():
             words = ["selectable", *words]
@@ -567,7 +463,7 @@ class Widget(metaclass=WidgetMeta):
             words.append("/".join(sorted(self.sizing())))
         return [*words, "widget"]
 
-    def _repr_attrs(self):
+    def _repr_attrs(self) -> dict[str, typing.Any]:
         return {}
 
     def keypress(self, size: tuple[()] | tuple[int] | tuple[int, int], key: str) -> str | None:
@@ -633,6 +529,48 @@ class Widget(metaclass=WidgetMeta):
                 LOGGER.debug(f"Widget {self!r} is not selectable")
         return False
 
+    def render(self, size: tuple[()] | tuple[int] | tuple[int, int], focus: bool = False) -> Canvas:
+        """Render widget and produce canvas
+
+        :param size: One of the following, *maxcol* and *maxrow* are integers > 0:
+
+            (*maxcol*, *maxrow*)
+              for box sizing -- the parent chooses the exact
+              size of this widget
+
+            (*maxcol*,)
+              for flow sizing -- the parent chooses only the
+              number of columns for this widget
+
+            ()
+              for fixed sizing -- this widget is a fixed size
+              which can't be adjusted by the parent
+        :type size: widget size
+        :param focus: set to ``True`` if this widget or one of its children is in focus
+        :type focus: bool
+
+        :returns: A :class:`Canvas` subclass instance containing the rendered content of this widget
+
+        :class:`Text` widgets return a :class:`TextCanvas` (arbitrary text and display attributes),
+        :class:`SolidFill` widgets return a :class:`SolidCanvas` (a single character repeated across the whole surface)
+        and container widgets return a :class:`CompositeCanvas` (one or more other canvases arranged arbitrarily).
+
+        If *focus* is ``False``, the returned canvas may not have a cursor position set.
+
+        There is some metaclass magic defined in the :class:`Widget` metaclass :class:`WidgetMeta`
+        that causes the result of this method to be cached by :class:`CanvasCache`.
+        Later calls will automatically look up the value in the cache first.
+
+        As a small optimization the class variable :attr:`ignore_focus`
+        may be defined and set to ``True`` if this widget renders the same
+        canvas regardless of the value of the *focus* parameter.
+
+        Any time the content of a widget changes it should call
+        :meth:`_invalidate` to remove any cached canvases, or the widget
+        may render the cached canvas instead of creating a new one.
+        """
+        raise NotImplementedError
+
 
 class FlowWidget(Widget):
     """
@@ -828,8 +766,8 @@ class WidgetWrapError(Exception):
     pass
 
 
-class WidgetWrap(delegate_to_widget_mixin("_wrapped_widget")):
-    def __init__(self, w: Widget):
+class WidgetWrap(delegate_to_widget_mixin("_wrapped_widget"), typing.Generic[WrappedWidget]):
+    def __init__(self, w: WrappedWidget) -> None:
         """
         w -- widget to wrap, stored as self._w
 
@@ -854,11 +792,11 @@ class WidgetWrap(delegate_to_widget_mixin("_wrapped_widget")):
         self._wrapped_widget = w
 
     @property
-    def _w(self) -> Widget:
+    def _w(self) -> WrappedWidget:
         return self._wrapped_widget
 
     @_w.setter
-    def _w(self, new_widget: Widget) -> None:
+    def _w(self, new_widget: WrappedWidget) -> None:
         """
         Change the wrapped widget.  This is meant to be called
         only by subclasses.
@@ -878,7 +816,7 @@ class WidgetWrap(delegate_to_widget_mixin("_wrapped_widget")):
         self._wrapped_widget = new_widget
         self._invalidate()
 
-    def _set_w(self, w):
+    def _set_w(self, w: WrappedWidget) -> None:
         """
         Change the wrapped widget.  This is meant to be called
         only by subclasses.
